import {RcsbFv3DCustom} from "../../RcsbFv3D/RcsbFv3DCustom";
import {RcsbFvStructureConfigInterface} from "../../RcsbFvStructure/RcsbFvStructure";
import {
    CustomViewInterface,
    FeatureBlockInterface
} from "../../RcsbFvSequence/SequenceViews/CustomView/CustomView";
import {
    LoadMethod,
    LoadMolstarInterface, LoadMolstarReturnType
} from "../../RcsbFvStructure/StructureViewers/MolstarViewer/MolstarActionManager";
import {ViewerProps} from "@rcsb/rcsb-molstar/build/src/viewer";

import {
    fetchHtpDescriptor,
    fetchPdbtmJsvLibDescriptor,
    htpDescriptorToTrackData,
    jsvLibDescriptorToTrackData
} from "./UniTmpHelper";
import { updateSiteColors } from "./UniTmpColor";
import { fvConfig, rowConfig } from "./FeatureViewConfig";


const side1 = "Inside";
const htpEntry = "SC5AA_HUMAN";
const pdbtmEntry = "7ynk_A";




const block: FeatureBlockInterface<LoadMolstarInterface<unknown,unknown>,LoadMolstarReturnType> = {
    blockId:"MyBlock_1",
    featureViewConfig: [fvConfig]
};

const customConfig: CustomViewInterface<LoadMolstarInterface<unknown,unknown>,LoadMolstarReturnType> = {
    blockConfig:[block]
}

const sequenceConfig = {
    title: "UniTmp example",
    subtitle: `HTP entry ${htpEntry}`,
    config: customConfig
};

const molstarConfig: RcsbFvStructureConfigInterface<LoadMolstarInterface<unknown,unknown>,{viewerProps:Partial<ViewerProps>}> = {
    loadConfig: {
        loadMethod: LoadMethod.loadPdbId,
        loadParams: {
            entryId: "7ynk",
            id:"7ynk_model"
        }
    },
    structureViewerConfig: {
        viewerProps:{
            showImportControls: true,
            showSessionControls: false
        }
    }
};

document.addEventListener("DOMContentLoaded", async function(event) {
    updateSiteColors(side1);
    const htpDescriptor = await fetchHtpDescriptor(htpEntry);
    const jsvlibDescriptor = await fetchPdbtmJsvLibDescriptor(pdbtmEntry);

    console.log(htpDescriptor);

    let sequence = htpDescriptor.Sequence.Seq;
    sequence = sequence.replace(/\s+/g, '');

    fvConfig.boardConfig.length = parseInt(htpDescriptor.Sequence['@attributes'].Length);
    fvConfig.rowConfig[0].rowTitle = `${htpDescriptor['@attributes'].id} sequence`;
    rowConfig[0].trackData![0].value = sequence;
    rowConfig.push(...htpDescriptorToTrackData(htpDescriptor));

    rowConfig.push(...jsvLibDescriptorToTrackData(jsvlibDescriptor));


    const panel3d = new RcsbFv3DCustom({
        elementId: "null",
        structurePanelConfig: molstarConfig,
        sequencePanelConfig: sequenceConfig
    });
    panel3d.render();
});
